from allauth.account.signals import user_logged_in
# from django.contrib.auth.signals import user_logged_in
from django.dispatch import receiver
from allauth.socialaccount.signals import pre_social_login


@receiver(pre_social_login)
def create_jwt_token(sender, request, sociallogin, **kwargs):
    print('----------------- create_jwt_token')
    print('sender %s' % sender)
    print('user %s' % request)
    print('sociallogin %s' % sociallogin)
    print('----------------- END create_jwt_token')
    # dig into the sociallogin object to find the new access token.


@receiver(user_logged_in)
def post_login(sender, user, request, **kwargs):
    print('---------------- post_login')
    print('sender: %s' % sender)
    print('user: %s' % user)
    print('request: %s' % request)
    print('---------------- END post_login')



from django.urls import path
from cbauth.views import (
    APIMeView,
    APILoginView,
    APILogoutView,
    APIPasswordResetView,
    APIPasswordResetConfirmView
)
from rest_framework_simplejwt.views import (TokenObtainPairView,
                                            TokenRefreshView,
                                            TokenVerifyView)


urlpatterns = [
    path('me/', APIMeView.as_view(), name='api_me'),
    path('login/', APILoginView.as_view(), name='api_login'),
    path('logout/', APILogoutView.as_view(), name='api_logout'),

    path('token/', TokenObtainPairView.as_view(),
         name='token_obtain_pair'),
    path('token/refresh/', TokenRefreshView.as_view(),
         name='token_refresh'),
    path('token/verify/', TokenVerifyView.as_view(),
         name='token_verify'),

    path('password_reset',
         APIPasswordResetView.as_view(),
         name='api_password_reset'),
    path('reset/<uidb64>/<token>/',
         APIPasswordResetConfirmView.as_view(),
         name='api_password_reset_confirm'),
]



from django.conf import settings
from django.urls import path
from django.contrib.auth.decorators import login_required
from django.views.generic import TemplateView
from django.contrib.auth.views import (
    LoginView,
    LogoutView
)

urlpatterns = []


if getattr(settings, 'CB_INCLUDE_MAIN_VIEW', False):
    main_view = login_required(TemplateView.as_view(template_name="main.html"))
    urlpatterns += [
        path('', main_view, name='auth_main'),
    ]

if getattr(settings, 'CB_INCLUDE_TEMPLATE_VIEWS', False):
    from cbauth.urls.pages import urlpatterns as views_urls
    urlpatterns += views_urls


if getattr(settings, 'CB_INCLUDE_API_VIEWS', True):
    from cbauth.urls.api import urlpatterns as api_urls
    urlpatterns += api_urls



from django.contrib.auth import get_user_model, authenticate
from rest_framework import serializers, exceptions

User = get_user_model()


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ('id', 'username', 'email',)
        read_only_fields = ('email', )





from django.urls import path
from django.urls import reverse_lazy
from django.contrib.auth.views import (
    LoginView,
    LogoutView,
    PasswordChangeView,
    PasswordChangeDoneView,
    PasswordResetView,
    PasswordResetDoneView,
    PasswordResetConfirmView,
    PasswordResetCompleteView,
)
from django.contrib.auth.decorators import login_required
from django.views.generic import TemplateView


urlpatterns = [
    path('login', LoginView.as_view(template_name="auth.login.html"),
         name='auth_login'),
    path('logout', LogoutView.as_view(next_page='/'), name='auth_logout'),
    path('', login_required(TemplateView.as_view(template_name="auth.main.html")),
         name='auth_main'),

    # Views to change password as logged-in
    path('password_change/', PasswordChangeView.as_view(), name='auth_password_change'),
    path('password_change/done/', PasswordChangeDoneView.as_view(), name='auth_password_change_done'),

    # Views to reset a password.
    path('password_reset/',
         PasswordResetView.as_view(
             email_template_name='auth.password_reset_email.html',
             template_name='auth.password_reset_form.html',
             success_url=reverse_lazy('auth_password_reset_done')
         ),
         name='auth_password_reset'),
    path('password_reset/done/',
         PasswordResetDoneView.as_view(template_name='auth.password_reset_done.html'),
         name='auth_password_reset_done'),
    path('reset/<uidb64>/<token>/',
         PasswordResetConfirmView.as_view(
             template_name='auth.password_reset_confirm.html',
             success_url=reverse_lazy('auth_password_reset_complete')
         ),
         name='auth_password_reset_confirm'),
    path('reset/done/',
         PasswordResetCompleteView.as_view(template_name='auth.password_reset_complete.html'),
         name='auth_password_reset_complete'),
]



from setuptools import setup, find_packages


packages = find_packages()
install_requires = [
    'djangorestframework',
    'djangorestframework_simplejwt',
    'django-allauth',
]


setup(name='cbauth',
      version='1.0.6',
      description='Django simplified version of authorization with template views and RESTFul API',
      long_description='',
      classifiers=[
          "Programming Language :: Python",
      ],
      author='code.bo',
      author_email='info@code.bo',
      url='https://github.com/codebolab/auth',
      packages=packages,
      include_package_data=True,
      package_data={
          '': [
              'cbauth/templates/*.html'
          ],
      },
      install_requires=install_requires,
      zip_safe=False)



# -*- coding: utf-8 -*-
import re
import logging

from django.urls import reverse
from django.test import TestCase
from django.test.client import Client
from django.core import mail
from django.urls import reverse_lazy


class TestBase(TestCase):
    def setUp(self):
        self.log = logging.getLogger('simple')
        self.client = Client()

    def request(self, method, path, data=None, **kwargs):
        if self.secure:
            kwargs['wsgi.url_scheme'] = 'https'

        CALLS = {
            'get': self.client.get,
            'post': self.client.post,
            'put': self.client.put,
            'delete': self.client.delete,
        }

        method = method.lower()
        if method not in CALLS:
            raise ValueError()

        call = CALLS.get(method)
        return call(path, data=data, **kwargs)


class TestLogin(TestBase):
    login_data = {
        'username': 'fakeuser',
        'password': 'fakepass'
    }
    secure = False
    fixtures = ['auth.json']

    def setUp(self):
        TestBase.setUp(self)

        self.login_url = reverse('admin:index')
        self.logout_url = reverse('admin:logout')

    def test_get_token(self):
        response = self.request('post', reverse('token_obtain_pair'),
                                data=self.login_data)
        self.log.info('response: %s ' % response)

    def test_login(self):
        response = self.request('post', reverse('auth_login'),
                                data=self.login_data)
        assert response.status_code == 302

    def test_get_login(self):
        response = self.request('get', reverse('auth_login'))
        self.log.info("login status code: %s" % response.content)
        self.log.info("response: %s " % response)
        self.log.info("login status code: %s" % response.status_code)
        assert response.status_code == 200

    def test_on_invalid_data(self):
        data = {
            'username': 'fakeuser',
            'password': 'invalidpass'
        }
        response = self.request('post', reverse('auth_login'),
                                data=data)
        self.assertEquals(response.status_code, 200)

    def test_login_fail_when_empty(self):
        data = {
            'username': 'fakeuser',
            'password': ''
        }
        response = self.request('post', reverse('auth_login'),
                                data=data)
        self.log.info('response: %s ' % response)
        self.assertEquals(response.status_code, 200)


class TestAPI(TestBase):
    login_data = {
        'username': 'fakeuser',
        'password': 'fakepass'
    }
    secure = False
    fixtures = ['auth.json']

    def test_login(self):
        response = self.request('post', reverse('api_login'),
                                data=self.login_data)
        self.log.info('response.class: %s ' % response.__class__)
        self.log.info('response.data: %s ' % response.data)
        self.log.info('response.status_code: %s ' % response.status_code)
        self.assertEquals(response.status_code, 200)
        self.assertIn('username', response.data)
        self.assertIn('access', response.data)
        self.assertIn('refresh', response.data)

    def test_login_invalid(self):
        data = {
            'username': 'fakeuser',
            'password': 'invalidpass'
        }
        response = self.request('post', reverse('api_login'), data=data)
        self.log.info('response.class: %s ' % response.__class__)
        self.log.info('response.data: %s ' % response.data)
        self.log.info('response.status_code: %s ' % response.status_code)
        self.assertEquals(response.status_code, 400)

    def test_me(self):
        # Calls fail
        response = self.request('get', reverse('api_me'))
        self.assertEquals(response.status_code, 403)

        # Calls login
        response = self.request('post', reverse('api_login'),
                                data=self.login_data)

        access_token = response.data.get('access')
        refresh_token = response.data.get('refresh')

        self.log.info("access_token: %s " % access_token)
        self.log.info("refresh_token: %s " % refresh_token)

        # Access fail without token after calling login
        response = self.request('get', reverse('api_me'))
        self.assertEquals(response.status_code, 403)

        # Access with token
        headers = {'HTTP_AUTHORIZATION': 'Bearer %s' % access_token}
        response = self.request('get', reverse('api_me'), **headers)
        self.log.info("response: %s " % response)
        self.log.info("response.data: %s " % response.data)
        self.assertEquals(response.status_code, 200)

    def test_logout(self):
        # Calls login
        response = self.request('post', reverse('api_login'),
                                data=self.login_data)

        access_token = response.data.get('access')
        # refresh_token = response.data.get('refresh')

        headers = {'HTTP_AUTHORIZATION': 'Bearer %s' % access_token}

        response = self.request('get', reverse('api_me'), **headers)
        self.log.info("response: %s " % response)
        self.log.info("response.data: %s " % response.data)
        self.assertEquals(response.status_code, 200)

        response = self.request('post', reverse('api_logout'), **headers)
        self.log.info("response: %s " % response)
        self.log.info("response.data: %s " % response.data)
        self.assertEquals(response.status_code, 200)

        response = self.request('get', reverse('api_me'), **headers)
        self.log.info("response: %s " % response)
        self.log.info("response.data: %s " % response.data)
        self.assertNotEquals(response.status_code, 200)

    def test_password_reset(self):
        pass_data = {
            'email': 'fakeuser@email.com',
        }
        response = self.request('post', reverse('api_password_reset'),
                                data=pass_data)
        self.log.info("response status: %s " % response.status_code)
        self.log.info("response: %s " % response)
        self.log.info("response.content: %s " % response.content)
        assert response.status_code == 200

    def test_password_reset_invalid(self):
        pass_data = {
            'email': 'fakeuser@email',
        }
        response = self.request('post', reverse('api_password_reset'),
                                data=pass_data)
        self.log.info("response status: %s " % response.status_code)
        assert response.status_code == 400

        pass_data = {
            'email': '',
        }
        response = self.request('post', reverse('api_password_reset'),
                                data=pass_data)
        self.log.info("response status: %s " % response.status_code)
        assert response.status_code == 400

    def _test_confirm_start_parts(self):
        # Start by creating the email
        self.client.post(reverse('api_password_reset'),
                         {'email': 'fakeuser@email.com'})
        self.assertEqual(len(mail.outbox), 1)
        url, path = self._read_signup_email(mail.outbox[0])

        if path.endswith('/'):
            path = path[:-1]
        parts = path.split('/')

        return parts[-2], parts[-1]

    def _read_signup_email(self, email):
        urlmatch = re.search(r"https?://[^/]*(/.*reset/\S*)", email.body)
        self.assertIsNotNone(urlmatch, "No URL found in sent email")
        return urlmatch.group(), urlmatch.groups()[0]

    def test_password_reset_confirm(self):
        uidb64, token = self._test_confirm_start_parts()
        kwargs = {
            'uidb64': uidb64,
            'token': token
        }
        api_path = reverse_lazy('api_password_reset_confirm', kwargs=kwargs)
        self.log.info("api_path: %s" % api_path)
        self.log.info("kwargs: %s " % kwargs)

        response = self.request('get', api_path)
        self.log.info(response.status_code)
        self.log.info(response.content)
        assert response.status_code == 200

        data = {
            'new_password1': 'anewpassword',
            'new_password2': 'anewpassword'
        }
        response = self.request('post', api_path, data)
        assert response.status_code == 200

    def test_password_reset_confirm_invalid(self):
        kwargs = {
            'uidb64': "invalid",
            'token': "invalidtoken",
        }
        api_path = reverse_lazy('api_password_reset_confirm', kwargs=kwargs)
        response = self.request('get', api_path)

        self.log.info(response.status_code)
        self.log.info(response.content)

        assert response.status_code == 403

    def test_password_reset_confirm_invalid_data(self):
        uidb64, token = self._test_confirm_start_parts()
        kwargs = {
            'uidb64': uidb64,
            'token': token
        }
        api_path = reverse_lazy('api_password_reset_confirm', kwargs=kwargs)

        data = {
            'new_password1': 'anewpasswordasdf',
            'new_password2': 'anewpassword'
        }
        response = self.request('post', api_path, data)
        assert response.status_code == 400



import logging

from django.conf import settings
from django.contrib.auth import login, logout
from django.contrib.auth.forms import AuthenticationForm
from django.views.generic.edit import FormMixin
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.tokens import default_token_generator
from django.contrib.auth.forms import (
    PasswordResetForm
)
from django.contrib.auth.views import (
    PasswordResetConfirmView
)


from rest_framework import generics
from rest_framework.response import Response
from rest_framework.permissions import IsAuthenticatedOrReadOnly
from rest_framework import permissions, status
from rest_framework_simplejwt.tokens import RefreshToken

from django.utils.decorators import method_decorator
from django.views.decorators.cache import never_cache
from django.views.decorators.debug import sensitive_post_parameters


logger = logging.getLogger('simple')


# RESTfull API Views
# ---------------------------------------------------------
class AuthMixin(object):
    def get(self, request, *args, **kwargs):
        response = {
            'message': _(u'Only POST method is allowed')
        }
        return Response(response, status=405)


class APIMeView(generics.GenericAPIView):
    permission_classes = (IsAuthenticatedOrReadOnly,)

    def get(self, request):
        """
        Returns user information
        """

        user = request.user
        if not user.is_anonymous:
            refresh = RefreshToken.for_user(user)
            data = {
                'username': user.username,
                'refresh': str(refresh),
                'access': str(refresh.access_token)
            }
            return Response(data, status=status.HTTP_200_OK)
        else:
            return Response({'message': 'no user'}, status=403)


class APILoginView(AuthMixin, generics.GenericAPIView):
    """
    Async Login API endpoint
    """

    permission_classes = (permissions.AllowAny,)

    def post(self, request, *args, **kwargs):
        form = AuthenticationForm(request, data=request.data)
        if form.is_valid():

            user = form.get_user()

            if getattr(settings, 'CB_GENERATE_COOKIE', False):
                login(request, user)

            refresh = RefreshToken.for_user(user)

            response = {
                'username': user.username,
                'refresh': str(refresh),
                'access': str(refresh.access_token)
            }

            return Response(response)
        else:
            response = {
                'errors': form.errors
            }
            return Response(response, status=400)


class APILogoutView(AuthMixin, generics.GenericAPIView):
    """
    Async Logout API endpoint
    """

    permission_classes = (permissions.AllowAny,)

    def post(self, request, *args, **kwargs):
        header = request.META.get('HTTP_AUTHORIZATION')

        if header is not None:
            try:
                token = RefreshToken(header)
                token.blacklist()
            except Exception as e:
                logger.error(e)

        logger.info(request.COOKIES)
        logout(request)

        return Response({"detail": _("Successfully logged out.")},
                        status=status.HTTP_200_OK)


class APIFormMixin(object):
    def get_form_kwargs(self):
        kwargs = super(APIFormMixin, self).get_form_kwargs()

        if self.request.method in ('POST', 'PUT'):
            kwargs['data'] = self.request.data

        return kwargs


class APIPasswordResetView(APIFormMixin, generics.GenericAPIView, FormMixin):
    permission_classes = (permissions.AllowAny,)
    from_email = None
    form_class = PasswordResetForm
    token_generator = default_token_generator
    email_template_name = 'auth.password_reset_email.html'
    subject_template_name = 'registration/password_reset_subject.txt'
    html_email_template_name = None

    def get_initial(self):
        return {}

    def post(self, request, *args, **kwargs):
        form = self.get_form()
        if form.is_valid():
            return self.form_valid(form)
        else:
            response = {
                'errors': form.errors
            }
            return Response(response, status=400)

    def form_valid(self, form):
        opts = {
            'use_https': self.request.is_secure(),
            'token_generator': self.token_generator,
            'from_email': self.from_email,
            'email_template_name': self.email_template_name,
            'subject_template_name': self.subject_template_name,
            'request': self.request,
            'html_email_template_name': self.html_email_template_name,
            'extra_email_context': None,
        }
        form.save(**opts)

        return Response({
            "detail": _(
                "We've emailed you instructions for setting your "
                "password, if an account exists with the email you entered. "
                "You should receive them shortly."
            )
        })


class APIPasswordResetConfirmView(APIFormMixin, generics.GenericAPIView,
                                  PasswordResetConfirmView):
    permission_classes = (permissions.AllowAny,)

    def get_initial(self):
        return {}

    @method_decorator(sensitive_post_parameters())
    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        logger.info(">>> dispatching")
        self.args = args
        self.kwargs = kwargs
        self.request = request
        self.headers = self.default_response_headers  # deprecate?

        assert 'uidb64' in kwargs and 'token' in kwargs

        self.validlink = False
        self.user = self.get_user(kwargs['uidb64'])

        if self.user is not None:
            token = kwargs['token']
            if self.token_generator.check_token(self.user, token):
                self.validlink = True
                return super().dispatch(request, *args, **kwargs)

        logger.info(">>> dispatching END")
        self.response = Response({
            "detail": _(
                "The password reset link was invalid, possibly because it has "
                "already been used.  Please request a new password reset."
            )
        }, status=403)
        request = self.initialize_request(request, *args, **kwargs)
        self.response = self.finalize_response(request, self.response, *args,
                                               **kwargs)
        return self.response

    def get(self, request, *args, **kwargs):
        return Response({
            "detail": _("The password reset link is valid.")
        }, status=200)

    def post(self, request, *args, **kwargs):
        form = self.get_form()
        if form.is_valid():
            return self.form_valid(form)
        else:
            response = {
                'errors': form.errors
            }
            return Response(response, status=400)

    def form_valid(self, form):
        form.save()
        return Response({
            'detail': _("Password changed successfully")
        }, status=200)
